@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include <telepathy-glib/util.h>
@@ -105,6 +106,10 @@ static void             contact_list_store_members_changed_cb        (EmpathyCon
 								      gchar                         *message,
 								      gboolean                       is_member,
 								      EmpathyContactListStore       *store);
+static void             contact_list_store_favourites_changed_cb     (EmpathyContactList            *list_iface,
+								      EmpathyContact                *contact,
+								      gboolean                       is_favourite,
+								      EmpathyContactListStore       *store);
 static void             contact_list_store_member_renamed_cb         (EmpathyContactList            *list_iface,
 								      EmpathyContact                *old_contact,
 								      EmpathyContact                *new_contact,
@@ -142,7 +147,8 @@ static void             contact_list_store_get_group                 (EmpathyCon
 								      const gchar                   *name,
 								      GtkTreeIter                   *iter_group_to_set,
 								      GtkTreeIter                   *iter_separator_to_set,
-								      gboolean                      *created);
+								      gboolean                      *created,
+								      gboolean                      is_fake_group);
 static gint             contact_list_store_state_sort_func           (GtkTreeModel                  *model,
 								      GtkTreeIter                   *iter_a,
 								      GtkTreeIter                   *iter_b,
@@ -191,6 +197,10 @@ contact_list_store_iface_setup (gpointer user_data)
 			  "members-changed",
 			  G_CALLBACK (contact_list_store_members_changed_cb),
 			  store);
+	g_signal_connect (priv->list,
+			  "favourites-changed",
+			  G_CALLBACK (contact_list_store_favourites_changed_cb),
+			  store);
 	g_signal_connect (priv->list,
 			  "groups-changed",
 			  G_CALLBACK (contact_list_store_groups_changed_cb),
@@ -337,6 +347,9 @@ contact_list_store_dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_members_changed_cb),
 					      object);
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_favourites_changed_cb),
+					      object);
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_groups_changed_cb),
 					      object);
@@ -591,7 +604,6 @@ empathy_contact_list_store_set_show_groups (EmpathyContactListStore *store,
 					    gboolean                 show_groups)
 {
 	EmpathyContactListStorePriv *priv;
-	GList                       *contacts, *l;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store));
 
@@ -603,19 +615,28 @@ empathy_contact_list_store_set_show_groups (EmpathyContactListStore *store,
 
 	priv->show_groups = show_groups;
 
-	/* Remove all contacts and add them back, not optimized but that's the
-	 * easy way :) */
-	gtk_tree_store_clear (GTK_TREE_STORE (store));
-	contacts = empathy_contact_list_get_members (priv->list);
-	for (l = contacts; l; l = l->next) {
-		contact_list_store_members_changed_cb (priv->list, l->data,
-						       NULL, 0, NULL,
-						       TRUE,
-						       store);
+	if (priv->setup_idle_id == 0) {
+		/* Remove all contacts and add them back, not optimized but
+		 * that's the easy way :)
+		 *
+		 * This is only done if there's not a pending setup idle
+		 * callback, otherwise it will race and the contacts will get
+		 * added twice */
+		GList *contacts, *l;
 
-		g_object_unref (l->data);
+		gtk_tree_store_clear (GTK_TREE_STORE (store));
+		contacts = empathy_contact_list_get_members (priv->list);
+		for (l = contacts; l; l = l->next) {
+			contact_list_store_members_changed_cb (priv->list,
+							       l->data,
+							       NULL, 0, NULL,
+							       TRUE,
+							       store);
+
+			g_object_unref (l->data);
+		}
+		g_list_free (contacts);
 	}
-	g_list_free (contacts);
 
 	g_object_notify (G_OBJECT (store), "show-groups");
 }
@@ -715,11 +736,13 @@ empathy_contact_list_store_row_separator_func (GtkTreeModel *model,
 gchar *
 empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 					    GtkTreePath  *path,
-					    gboolean     *path_is_group)
+					    gboolean     *path_is_group,
+					    gboolean     *is_fake_group)
 {
 	GtkTreeIter  parent_iter, iter;
 	gchar       *name = NULL;
 	gboolean     is_group;
+	gboolean     fake;
 
 	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
 
@@ -749,6 +772,7 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 		gtk_tree_model_get (model, &iter,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake,
 				    -1);
 		if (!is_group) {
 			g_free (name);
@@ -760,6 +784,9 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 		*path_is_group = TRUE;
 	}
 
+	if (is_fake_group != NULL)
+		*is_fake_group = fake;
+
 	return name;
 }
 
@@ -814,8 +841,9 @@ contact_list_store_setup (EmpathyContactListStore *store)
 		GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
 		G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
 		G_TYPE_STRING,        /* Name */
+		G_TYPE_UINT,          /* Presence type */
 		G_TYPE_STRING,        /* Status string */
-		G_TYPE_BOOLEAN,       /* Show status */
+		G_TYPE_BOOLEAN,       /* Compact view */
 		EMPATHY_TYPE_CONTACT, /* Contact type */
 		G_TYPE_BOOLEAN,       /* Is group */
 		G_TYPE_BOOLEAN,       /* Is active */
@@ -824,6 +852,7 @@ contact_list_store_setup (EmpathyContactListStore *store)
 		G_TYPE_BOOLEAN,       /* Can make audio calls */
 		G_TYPE_BOOLEAN,       /* Can make video calls */
 		EMPATHY_TYPE_CONTACT_LIST_FLAGS, /* Flags */
+		G_TYPE_BOOLEAN,       /* Is a fake group */
 	};
 
 	priv = GET_PRIV (store);
@@ -916,6 +945,25 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 	}
 }
 
+static void
+contact_list_store_favourites_changed_cb (EmpathyContactList      *list_iface,
+					  EmpathyContact          *contact,
+					  gboolean                 is_favourite,
+					  EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	priv = GET_PRIV (store);
+
+	DEBUG ("Contact %s (%d) is %s a favourite",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		is_favourite ? "now" : "no longer");
+
+	contact_list_store_remove_contact (store, contact);
+	contact_list_store_add_contact (store, contact);
+}
+
 static void
 contact_list_store_member_renamed_cb (EmpathyContactList      *list_iface,
 				      EmpathyContact          *old_contact,
@@ -968,6 +1016,27 @@ contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 	priv->show_active = show_active;
 }
 
+static void
+add_contact_to_store (GtkTreeStore *store,
+		      GtkTreeIter *iter,
+		      EmpathyContact *contact,
+		      EmpathyContactListFlags flags)
+{
+	gtk_tree_store_set (store, iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+			      empathy_contact_get_capabilities (contact) &
+			        EMPATHY_CAPABILITIES_AUDIO,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+			      empathy_contact_get_capabilities (contact) &
+			        EMPATHY_CAPABILITIES_VIDEO,
+			    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
+			    -1);
+}
+
 static void
 contact_list_store_add_contact (EmpathyContactListStore *store,
 				EmpathyContact          *contact)
@@ -977,6 +1046,7 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 	GList                      *groups = NULL, *l;
 	TpConnection               *connection;
 	EmpathyContactListFlags     flags = 0;
+	char                       *protocol_name;
 
 	priv = GET_PRIV (store);
 
@@ -994,68 +1064,67 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		flags = empathy_contact_manager_get_flags_for_connection (
 			EMPATHY_CONTACT_MANAGER (priv->list), connection);
 	}
-	/* If no groups just add it at the top level. */
+
+	tp_connection_parse_object_path (connection, &protocol_name, NULL);
+
 	if (!groups) {
-		GtkTreeModel *model = GTK_TREE_MODEL (store);
+		GtkTreeIter iter_group, *parent;
 
-		if (gtk_tree_model_get_iter_first (model, &iter)) do {
-			EmpathyContact *c;
+		parent = &iter_group;
 
-			gtk_tree_model_get (model, &iter,
-				EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &c,
-				-1);
+		if (!priv->show_groups) {
+			parent = NULL;
+		} else if (!tp_strdiff (protocol_name, "local-xmpp")) {
+			/* these are People Nearby */
+			contact_list_store_get_group (store,
+				EMPATHY_CONTACT_LIST_STORE_PEOPLE_NEARBY,
+				&iter_group, NULL, NULL, TRUE);
+		} else {
+			contact_list_store_get_group (store,
+				EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
+				&iter_group, NULL, NULL, TRUE);
+		}
 
-			if (c == contact) {
-				g_object_unref (c);
-				return;
-			}
-			if (c != NULL)
-				g_object_unref (c);
-		} while (gtk_tree_model_iter_next (model, &iter));
+		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
+					     parent, NULL);
 
-		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_AUDIO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_VIDEO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
-				    -1);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter,
+				      contact, flags);
 	}
 
+	g_free (protocol_name);
+
 	/* Else add to each group. */
 	for (l = groups; l; l = l->next) {
 		GtkTreeIter iter_group;
 
-		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL);
+		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL, FALSE);
 
 		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
 					     &iter_group, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_AUDIO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_VIDEO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
-				    -1);
+
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
 		g_free (l->data);
 	}
 	g_list_free (groups);
 
-	contact_list_store_contact_update (store, contact);
+#ifdef HAVE_FAVOURITE_CONTACTS
+	if (priv->show_groups &&
+	    empathy_contact_list_is_favourite (priv->list, contact)) {
+	/* Add contact to the fake 'Favorites' group */
+		GtkTreeIter iter_group;
 
+		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_FAVORITE,
+			&iter_group, NULL, NULL, TRUE);
+
+		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
+					     &iter_group, NULL);
+
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+	}
+#endif
+
+	contact_list_store_contact_update (store, contact);
 }
 
 static void
@@ -1214,8 +1283,11 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, pixbuf_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS, empathy_contact_get_status (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
+				    EMPATHY_CONTACT_LIST_STORE_COL_PRESENCE_TYPE,
+				      empathy_contact_get_presence (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
+				      empathy_contact_get_presence_message (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_COMPACT, priv->is_compact,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE, now_online,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
@@ -1389,7 +1461,8 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			      const gchar            *name,
 			      GtkTreeIter            *iter_group_to_set,
 			      GtkTreeIter            *iter_separator_to_set,
-			      gboolean               *created)
+			      gboolean               *created,
+			      gboolean               is_fake_group)
 {
 	EmpathyContactListStorePriv *priv;
 	GtkTreeModel                *model;
@@ -1420,6 +1493,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, TRUE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, is_fake_group,
 				    -1);
 
 		if (iter_group_to_set) {
@@ -1461,45 +1535,159 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 	}
 }
 
+static gint
+get_position (const char **strv,
+	      const char *str)
+{
+	int i;
+
+	for (i = 0; strv[i] != NULL; i++) {
+		if (!tp_strdiff (strv[i], str))
+			return i;
+	}
+
+	return -1;
+}
+
+static gint
+compare_separator_and_groups (gboolean is_separator_a,
+			      gboolean is_separator_b,
+			      const gchar *name_a,
+			      const gchar *name_b,
+			      EmpathyContact *contact_a,
+			      EmpathyContact *contact_b,
+			      gboolean fake_group_a,
+			      gboolean fake_group_b)
+{
+	/* these two lists are the sorted list of fake groups to include at the
+	 * top and bottom of the roster */
+	const char *top_groups[] = {
+		EMPATHY_CONTACT_LIST_STORE_FAVORITE,
+		NULL
+	};
+
+	const char *bottom_groups[] = {
+		EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
+		NULL
+	};
+
+	if (is_separator_a || is_separator_b) {
+		/* We have at least one separator */
+		if (is_separator_a) {
+			return -1;
+		} else if (is_separator_b) {
+			return 1;
+		}
+	}
+
+	/* One group and one contact */
+	if (!contact_a && contact_b) {
+		return 1;
+	} else if (contact_a && !contact_b) {
+		return -1;
+	} else if (!contact_a && !contact_b) {
+		gboolean a_in_top, b_in_top, a_in_bottom, b_in_bottom;
+
+		a_in_top = fake_group_a &&
+			tp_strv_contains (top_groups, name_a);
+		b_in_top = fake_group_b &&
+			tp_strv_contains (top_groups, name_b);
+		a_in_bottom = fake_group_a &&
+			tp_strv_contains (bottom_groups, name_a);
+		b_in_bottom = fake_group_b &&
+			tp_strv_contains (bottom_groups, name_b);
+
+		if (a_in_top && b_in_top) {
+			/* compare positions */
+			return CLAMP (get_position (top_groups, name_a) -
+				      get_position (top_groups, name_b),
+				      -1, 1);
+		} else if (a_in_bottom && b_in_bottom) {
+			/* compare positions */
+			return CLAMP (get_position (bottom_groups, name_a) -
+				      get_position (bottom_groups, name_b),
+				      -1, 1);
+		} else if (a_in_top || b_in_bottom) {
+			return -1;
+		} else if (b_in_top || a_in_bottom) {
+			return 1;
+		} else {
+			return g_utf8_collate (name_a, name_b);
+		}
+	}
+
+	/* Two contacts, ordering depends of the sorting policy */
+	return 0;
+}
+
+static gint
+contact_list_store_contact_sort (EmpathyContact *contact_a,
+				 EmpathyContact *contact_b)
+{
+	TpAccount *account_a, *account_b;
+	gint ret_val;
+
+	/* alias */
+	ret_val = g_utf8_collate (empathy_contact_get_name (contact_a),
+				  empathy_contact_get_name (contact_b));
+
+	if (ret_val != 0)
+		goto out;
+
+	/* identifier */
+	ret_val = g_utf8_collate (empathy_contact_get_id (contact_a),
+				  empathy_contact_get_id (contact_b));
+
+	if (ret_val != 0)
+		goto out;
+
+	account_a = empathy_contact_get_account (contact_a);
+	account_b = empathy_contact_get_account (contact_b);
+
+	/* protocol */
+	ret_val = strcmp (tp_account_get_protocol (account_a),
+			  tp_account_get_protocol (account_a));
+
+	if (ret_val != 0)
+		goto out;
+
+	/* account ID */
+	ret_val = strcmp (tp_proxy_get_object_path (account_a),
+			  tp_proxy_get_object_path (account_a));
+
+out:
+	return ret_val;
+}
+
 static gint
 contact_list_store_state_sort_func (GtkTreeModel *model,
 				    GtkTreeIter  *iter_a,
 				    GtkTreeIter  *iter_b,
 				    gpointer      user_data)
 {
-	gint            ret_val = 0;
+	gint            ret_val;
 	gchar          *name_a, *name_b;
 	gboolean        is_separator_a, is_separator_b;
 	EmpathyContact *contact_a, *contact_b;
+	gboolean       fake_group_a, fake_group_b;
 
 	gtk_tree_model_get (model, iter_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	/* Separator or group? */
-	if (is_separator_a || is_separator_b) {
-		if (is_separator_a) {
-			ret_val = -1;
-		} else if (is_separator_b) {
-			ret_val = 1;
-		}
-	} else if (!contact_a && contact_b) {
-		ret_val = 1;
-	} else if (contact_a && !contact_b) {
-		ret_val = -1;
-	} else if (!contact_a && !contact_b) {
-		/* Handle groups */
-		ret_val = g_utf8_collate (name_a, name_b);
-	}
+	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 
-	if (ret_val) {
+	if (ret_val != 0) {
 		goto free_and_out;
 	}
 
@@ -1511,8 +1699,8 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 		empathy_contact_get_presence (EMPATHY_CONTACT (contact_b)));
 
 	if (ret_val == 0) {
-		/* Fallback: compare by name */
-		ret_val = g_utf8_collate (name_a, name_b);
+		/* Fallback: compare by name et al. */
+		ret_val = contact_list_store_contact_sort (contact_a, contact_b);
 	}
 
 free_and_out:
@@ -1538,38 +1726,28 @@ contact_list_store_name_sort_func (GtkTreeModel *model,
 {
 	gchar         *name_a, *name_b;
 	EmpathyContact *contact_a, *contact_b;
-	gboolean       is_separator_a, is_separator_b;
+	gboolean       is_separator_a = FALSE, is_separator_b = FALSE;
 	gint           ret_val;
+	gboolean       fake_group_a, fake_group_b;
 
 	gtk_tree_model_get (model, iter_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	/* If contact is NULL it means it's a group. */
+	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 
-	if (is_separator_a || is_separator_b) {
-		if (is_separator_a) {
-			ret_val = -1;
-		} else if (is_separator_b) {
-			ret_val = 1;
-		}
-	} else if (!contact_a && contact_b) {
-		ret_val = 1;
-	} else if (contact_a && !contact_b) {
-		ret_val = -1;
-	} else {
-		ret_val = g_utf8_collate (name_a, name_b);
-	}
-
-	g_free (name_a);
-	g_free (name_b);
+	if (ret_val == 0)
+		ret_val = contact_list_store_contact_sort (contact_a, contact_b);
 
 	if (contact_a) {
 		g_object_unref (contact_a);
@@ -1663,7 +1841,7 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	gtk_tree_store_set (GTK_TREE_STORE (store), iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf_status,
 			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
-			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_COMPACT, priv->is_compact,
 			    -1);
 
 	return FALSE;
