@@ -219,7 +219,7 @@ contact_list_view_dnd_get_contact_free (DndGetContactData *data)
 }
 
 static void
-contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
+contact_list_view_drag_got_contact (TpConnection            *connection,
 				    EmpathyContact          *contact,
 				    const GError            *error,
 				    gpointer                 user_data,
@@ -265,8 +265,8 @@ contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
 
 static gboolean
 group_can_be_modified (const gchar *name,
-		       gboolean is_fake_group,
-		       gboolean adding)
+		       gboolean     is_fake_group,
+		       gboolean     adding)
 {
 	/* Real groups can always be modified */
 	if (!is_fake_group)
@@ -293,7 +293,7 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 {
 	EmpathyContactListViewPriv *priv;
 	TpAccountManager           *account_manager;
-	EmpathyTpContactFactory    *factory = NULL;
+        TpConnection               *connection;
 	TpAccount                  *account;
 	DndGetContactData          *data;
 	GtkTreePath                *source_path;
@@ -342,20 +342,15 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 		account = tp_account_manager_ensure_account (account_manager, account_id);
 	}
 	if (account) {
-		TpConnection *connection;
-
 		connection = tp_account_get_connection (account);
-		if (connection) {
-			factory = empathy_tp_contact_factory_dup_singleton (connection);
-		}
 	}
-	g_object_unref (account_manager);
 
-	if (!factory) {
-		DEBUG ("Failed to get factory for account '%s'", account_id);
+	if (!connection) {
+		DEBUG ("Failed to get connection for account '%s'", account_id);
 		success = FALSE;
 		g_free (new_group);
 		g_free (old_group);
+		g_object_unref (account_manager);
 		return FALSE;
 	}
 
@@ -366,12 +361,12 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 
 	/* FIXME: We should probably wait for the cb before calling
 	 * gtk_drag_finish */
-	empathy_tp_contact_factory_get_from_id (factory, contact_id,
+	empathy_tp_contact_factory_get_from_id (connection, contact_id,
 						contact_list_view_drag_got_contact,
 						data, (GDestroyNotify) contact_list_view_dnd_get_contact_free,
 						G_OBJECT (view));
 	g_strfreev (strv);
-	g_object_unref (factory);
+	g_object_unref (account_manager);
 
 	return TRUE;
 }
@@ -864,9 +859,9 @@ contact_list_view_call_activated_cb (
 
 static void
 contact_list_view_cell_set_background (EmpathyContactListView *view,
-				       GtkCellRenderer       *cell,
-				       gboolean               is_group,
-				       gboolean               is_active)
+				       GtkCellRenderer        *cell,
+				       gboolean                is_group,
+				       gboolean                is_active)
 {
 	GdkColor  color;
 	GtkStyle *style;
@@ -896,10 +891,10 @@ contact_list_view_cell_set_background (EmpathyContactListView *view,
 }
 
 static void
-contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column,
-					 GtkCellRenderer       *cell,
-					 GtkTreeModel          *model,
-					 GtkTreeIter           *iter,
+contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn      *tree_column,
+					 GtkCellRenderer        *cell,
+					 GtkTreeModel           *model,
+					 GtkTreeIter            *iter,
 					 EmpathyContactListView *view)
 {
 	GdkPixbuf *pixbuf;
@@ -992,10 +987,10 @@ contact_list_view_audio_call_cell_data_func (
 }
 
 static void
-contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
-					 GtkCellRenderer       *cell,
-					 GtkTreeModel          *model,
-					 GtkTreeIter           *iter,
+contact_list_view_avatar_cell_data_func (GtkTreeViewColumn      *tree_column,
+					 GtkCellRenderer        *cell,
+					 GtkTreeModel           *model,
+					 GtkTreeIter            *iter,
 					 EmpathyContactListView *view)
 {
 	GdkPixbuf *pixbuf;
@@ -1023,10 +1018,10 @@ contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
 }
 
 static void
-contact_list_view_text_cell_data_func (GtkTreeViewColumn     *tree_column,
-				       GtkCellRenderer       *cell,
-				       GtkTreeModel          *model,
-				       GtkTreeIter           *iter,
+contact_list_view_text_cell_data_func (GtkTreeViewColumn      *tree_column,
+				       GtkCellRenderer        *cell,
+				       GtkTreeModel           *model,
+				       GtkTreeIter            *iter,
 				       EmpathyContactListView *view)
 {
 	gboolean is_group;
@@ -1041,10 +1036,10 @@ contact_list_view_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 }
 
 static void
-contact_list_view_expander_cell_data_func (GtkTreeViewColumn     *column,
-					   GtkCellRenderer       *cell,
-					   GtkTreeModel          *model,
-					   GtkTreeIter           *iter,
+contact_list_view_expander_cell_data_func (GtkTreeViewColumn      *column,
+					   GtkCellRenderer        *cell,
+					   GtkTreeModel           *model,
+					   GtkTreeIter            *iter,
 					   EmpathyContactListView *view)
 {
 	gboolean is_group;
@@ -1076,9 +1071,9 @@ contact_list_view_expander_cell_data_func (GtkTreeViewColumn     *column,
 
 static void
 contact_list_view_row_expand_or_collapse_cb (EmpathyContactListView *view,
-					     GtkTreeIter           *iter,
-					     GtkTreePath           *path,
-					     gpointer               user_data)
+					     GtkTreeIter            *iter,
+					     GtkTreePath            *path,
+					     gpointer                user_data)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	GtkTreeModel               *model;
@@ -1102,9 +1097,9 @@ contact_list_view_row_expand_or_collapse_cb (EmpathyContactListView *view,
 }
 
 static void
-contact_list_view_row_has_child_toggled_cb (GtkTreeModel          *model,
-					    GtkTreePath           *path,
-					    GtkTreeIter           *iter,
+contact_list_view_row_has_child_toggled_cb (GtkTreeModel           *model,
+					    GtkTreePath            *path,
+					    GtkTreeIter            *iter,
 					    EmpathyContactListView *view)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
