@@ -26,6 +26,7 @@
 
 #include <glib/gi18n.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
 #include <geoclue/geoclue-master.h>
@@ -35,10 +36,8 @@
 #include "empathy-location-manager.h"
 #include "empathy-conf.h"
 
-#include "libempathy/empathy-account-manager.h"
 #include "libempathy/empathy-enum-types.h"
 #include "libempathy/empathy-location.h"
-#include "libempathy/empathy-tp-contact-factory.h"
 #include "libempathy/empathy-utils.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_LOCATION
@@ -63,7 +62,7 @@ typedef struct {
     GeoclueAddress *gc_address;
 
     gboolean reduce_accuracy;
-    EmpathyAccountManager *account_manager;
+    TpAccountManager *account_manager;
 
     /* The idle id for publish_on_idle func */
     guint timeout_id;
@@ -98,6 +97,8 @@ static void
 location_manager_dispose (GObject *object)
 {
   EmpathyLocationManagerPriv *priv = GET_PRIV (object);
+  void (*dispose) (GObject *) =
+    G_OBJECT_CLASS (empathy_location_manager_parent_class)->dispose;
 
   if (priv->account_manager != NULL)
   {
@@ -129,7 +130,8 @@ location_manager_dispose (GObject *object)
     priv->location = NULL;
   }
 
-  G_OBJECT_CLASS (empathy_location_manager_parent_class)->finalize (object);
+  if (dispose != NULL)
+    dispose (object);
 }
 
 static void
@@ -179,6 +181,16 @@ empathy_location_manager_class_init (EmpathyLocationManagerClass *class)
   g_type_class_add_private (object_class, sizeof (EmpathyLocationManagerPriv));
 }
 
+static void
+publish_location_cb (TpConnection *connection,
+                     const GError *error,
+                     gpointer user_data,
+                     GObject *weak_object)
+{
+  if (error != NULL)
+      DEBUG ("Error setting location: %s", error->message);
+}
+
 static void
 publish_location (EmpathyLocationManager *self,
     TpConnection *conn,
@@ -188,7 +200,6 @@ publish_location (EmpathyLocationManager *self,
   guint connection_status = -1;
   gboolean can_publish;
   EmpathyConf *conf = empathy_conf_get ();
-  EmpathyTpContactFactory *factory;
 
   if (!conn)
     return;
@@ -212,9 +223,46 @@ publish_location (EmpathyLocationManager *self,
       (g_hash_table_size (priv->location) == 0 ? "empty" : ""),
       conn);
 
-  factory = empathy_tp_contact_factory_dup_singleton (conn);
-  empathy_tp_contact_factory_set_location (factory, priv->location);
-  g_object_unref (factory);
+  tp_cli_connection_interface_location_call_set_location (conn, -1,
+      priv->location, publish_location_cb, NULL, NULL, G_OBJECT (self));
+}
+
+typedef struct
+{
+  EmpathyLocationManager *self;
+  gboolean force_publication;
+} PublishToAllData;
+
+static void
+publish_to_all_am_prepared_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+  PublishToAllData *data = user_data;
+  GList *accounts, *l;
+  GError *error = NULL;
+
+  if (!tp_account_manager_prepare_finish (manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  accounts = tp_account_manager_get_valid_accounts (manager);
+  for (l = accounts; l; l = l->next)
+    {
+      TpConnection *conn = tp_account_get_connection (TP_ACCOUNT (l->data));
+
+      if (conn != NULL)
+        publish_location (data->self, conn, data->force_publication);
+    }
+  g_list_free (accounts);
+
+out:
+  g_object_unref (data->self);
+  g_slice_free (PublishToAllData, data);
 }
 
 static void
@@ -222,16 +270,14 @@ publish_to_all_connections (EmpathyLocationManager *self,
     gboolean force_publication)
 {
   EmpathyLocationManagerPriv *priv = GET_PRIV (self);
-  GList *connections = NULL, *l;
+  PublishToAllData *data;
 
-  connections = empathy_account_manager_dup_connections (priv->account_manager);
-  for (l = connections; l; l = l->next)
-    {
-      publish_location (self, l->data, force_publication);
-      g_object_unref (l->data);
-    }
-  g_list_free (connections);
+  data = g_slice_new0 (PublishToAllData);
+  data->self = g_object_ref (self);
+  data->force_publication = force_publication;
 
+  tp_account_manager_prepare_async (priv->account_manager, NULL,
+      publish_to_all_am_prepared_cb, data);
 }
 
 static gboolean
@@ -246,11 +292,19 @@ publish_on_idle (gpointer user_data)
 }
 
 static void
-new_connection_cb (EmpathyAccountManager *manager,
-    TpConnection *conn,
+new_connection_cb (TpAccount *account,
+    guint old_status,
+    guint new_status,
+    guint reason,
+    gchar *dbus_error_name,
+    GHashTable *details,
     gpointer *self)
 {
   EmpathyLocationManagerPriv *priv = GET_PRIV (self);
+  TpConnection *conn;
+
+  conn = tp_account_get_connection (account);
+
   DEBUG ("New connection %p", conn);
 
   /* Don't publish if it is already planned (ie startup) */
@@ -466,7 +520,7 @@ update_resources (EmpathyLocationManager *self)
    * being found as geoclue-manual report an empty address with
    * accuracy = NONE */
   if (!geoclue_master_client_set_requirements (priv->gc_client,
-          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, TRUE, priv->resources,
+          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, priv->resources,
           NULL))
     {
       DEBUG ("set_requirements failed");
@@ -625,6 +679,34 @@ accuracy_cb (EmpathyConf  *conf,
       initial_position_cb, manager);
 }
 
+static void
+account_manager_prepared_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  GList *accounts, *l;
+  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+  EmpathyLocationManager *self = user_data;
+  GError *error = NULL;
+
+  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  accounts = tp_account_manager_get_valid_accounts (account_manager);
+  for (l = accounts; l != NULL; l = l->next)
+    {
+      TpAccount *account = TP_ACCOUNT (l->data);
+
+      tp_g_signal_connect_object (account, "status-changed",
+          G_CALLBACK (new_connection_cb), self, 0);
+    }
+  g_list_free (accounts);
+}
+
 static void
 empathy_location_manager_init (EmpathyLocationManager *self)
 {
@@ -638,10 +720,10 @@ empathy_location_manager_init (EmpathyLocationManager *self)
       g_free, (GDestroyNotify) tp_g_value_slice_free);
 
   /* Setup account status callbacks */
-  priv->account_manager = empathy_account_manager_dup_singleton ();
-  g_signal_connect (priv->account_manager,
-    "new-connection",
-    G_CALLBACK (new_connection_cb), self);
+  priv->account_manager = tp_account_manager_dup ();
+
+  tp_account_manager_prepare_async (priv->account_manager, NULL,
+      account_manager_prepared_cb, self);
 
   /* Setup settings status callbacks */
   conf = empathy_conf_get ();
