@@ -54,6 +54,15 @@ contact_list_base_init (gpointer klass)
 	static gboolean initialized = FALSE;
 
 	if (!initialized) {
+		g_signal_new ("member-renamed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      G_TYPE_NONE,
+			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
+
 		g_signal_new ("members-changed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -64,6 +73,15 @@ contact_list_base_init (gpointer klass)
 			      5, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT,
 			      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
+		g_signal_new ("favourites-changed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      _empathy_marshal_VOID__OBJECT_BOOLEAN,
+			      G_TYPE_NONE,
+			      2, EMPATHY_TYPE_CONTACT, G_TYPE_BOOLEAN);
+
 		g_signal_new ("pendings-changed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -125,18 +143,6 @@ empathy_contact_list_get_members (EmpathyContactList *list)
 	return NULL;
 }
 
-EmpathyContactMonitor *
-empathy_contact_list_get_monitor (EmpathyContactList *list)
-{
-	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST (list), NULL);
-
-	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_monitor) {
-		return EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_monitor (list);
-	}
-
-	return NULL;
-}
-
 GList *
 empathy_contact_list_get_pendings (EmpathyContactList *list)
 {
@@ -240,3 +246,41 @@ empathy_contact_list_get_flags (EmpathyContactList *list)
 		return 0;
 	}
 }
+
+gboolean
+empathy_contact_list_is_favourite (EmpathyContactList *list,
+                                   EmpathyContact     *contact)
+{
+#if HAVE_FAVOURITE_CONTACTS
+	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->is_favourite) {
+		return EMPATHY_CONTACT_LIST_GET_IFACE (list)->is_favourite (
+			list, contact);
+	}
+#endif /* HAVE_FAVOURITE_CONTACTS */
+
+	return FALSE;
+}
+
+void
+empathy_contact_list_add_to_favourites (EmpathyContactList *list,
+                                        EmpathyContact     *contact)
+{
+#if HAVE_FAVOURITE_CONTACTS
+	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->add_favourite) {
+		EMPATHY_CONTACT_LIST_GET_IFACE (list)->add_favourite (list,
+			contact);
+	}
+#endif /* HAVE_FAVOURITE_CONTACTS */
+}
+
+void
+empathy_contact_list_remove_from_favourites (EmpathyContactList *list,
+                                             EmpathyContact     *contact)
+{
+#if HAVE_FAVOURITE_CONTACTS
+	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->remove_favourite) {
+		EMPATHY_CONTACT_LIST_GET_IFACE (list)->remove_favourite (list,
+			contact);
+	}
+#endif /* HAVE_FAVOURITE_CONTACTS */
+}
