@@ -84,6 +84,7 @@ const gchar * empathy_contact_get_id (EmpathyContact *contact);
 void empathy_contact_set_id (EmpathyContact *contact, const gchar *id);
 const gchar * empathy_contact_get_name (EmpathyContact *contact);
 void empathy_contact_set_name (EmpathyContact *contact, const gchar *name);
+void empathy_contact_set_alias (EmpathyContact *contact, const gchar *alias);
 EmpathyAvatar * empathy_contact_get_avatar (EmpathyContact *contact);
 void empathy_contact_set_avatar (EmpathyContact *contact,
     EmpathyAvatar *avatar);
@@ -110,10 +111,6 @@ gboolean empathy_contact_can_voip_audio (EmpathyContact *contact);
 gboolean empathy_contact_can_voip_video (EmpathyContact *contact);
 gboolean empathy_contact_can_send_files (EmpathyContact *contact);
 gboolean empathy_contact_can_use_stream_tube (EmpathyContact *contact);
-
-void empathy_contact_load_avatar_data (EmpathyContact *contact,
-    const guchar *data, const gsize len, const gchar *format,
-    const gchar *token);
 gboolean empathy_contact_load_avatar_cache (EmpathyContact *contact,
     const gchar *token);
 
@@ -135,6 +132,8 @@ GHashTable * empathy_contact_get_location (EmpathyContact *contact);
 gboolean empathy_contact_equal (gconstpointer contact1,
     gconstpointer contact2);
 
+EmpathyContact *empathy_contact_dup_from_tp_contact (TpContact *tp_contact);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_CONTACT_H__ */
