@@ -185,7 +185,7 @@ empathy_dispatch_operation_invalidated (TpProxy *proxy, guint domain,
 }
 
 static void
-dispatcher_operation_got_contact_cb (EmpathyTpContactFactory *factory,
+dispatcher_operation_got_contact_cb (TpConnection *connection,
                                      EmpathyContact *contact,
                                      const GError *error,
                                      gpointer user_data,
@@ -219,7 +219,6 @@ dispatch_operation_connection_ready (TpConnection *connection,
 {
   EmpathyDispatchOperation *self = EMPATHY_DISPATCH_OPERATION (user_data);
   EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
-  EmpathyTpContactFactory *factory;
   TpHandle handle;
 
   if (error != NULL)
@@ -231,12 +230,9 @@ dispatch_operation_connection_ready (TpConnection *connection,
 
   handle = tp_channel_get_handle (priv->channel, NULL);
 
-  factory = empathy_tp_contact_factory_dup_singleton (priv->connection);
-
-  empathy_tp_contact_factory_get_from_handle (factory, handle,
+  empathy_tp_contact_factory_get_from_handle (priv->connection, handle,
       dispatcher_operation_got_contact_cb, NULL, NULL, G_OBJECT (self));
 
-  g_object_unref (factory);
 out:
   g_object_unref (self);
 }
@@ -436,6 +432,18 @@ empathy_dispatch_operation_set_status (EmpathyDispatchOperation *self,
     }
 }
 
+static void
+channel_wrapper_ready (EmpathyDispatchOperation *self)
+{
+  EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
+
+  g_signal_handler_disconnect (priv->channel_wrapper, priv->ready_handler);
+  priv->ready_handler = 0;
+
+  empathy_dispatch_operation_set_status (self,
+    EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
+}
+
 static void
 empathy_dispatcher_operation_tp_chat_ready_cb (GObject *object,
   GParamSpec *spec, gpointer user_data)
@@ -446,11 +454,18 @@ empathy_dispatcher_operation_tp_chat_ready_cb (GObject *object,
   if (!empathy_tp_chat_is_ready (EMPATHY_TP_CHAT (priv->channel_wrapper)))
     return;
 
-  g_signal_handler_disconnect (priv->channel_wrapper, priv->ready_handler);
-  priv->ready_handler = 0;
+  channel_wrapper_ready (self);
+}
 
-  empathy_dispatch_operation_set_status (self,
-    EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
+static void
+call_status_changed_cb (EmpathyTpCall *call,
+    GParamSpec *spec,
+    EmpathyDispatchOperation *self)
+{
+  if (empathy_tp_call_get_status (call) <= EMPATHY_TP_CALL_STATUS_READYING)
+    return;
+
+  channel_wrapper_ready (self);
 }
 
 static void
@@ -491,8 +506,22 @@ empathy_dispatch_operation_channel_ready_cb (TpChannel *channel,
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
     {
+      gboolean requested;
       EmpathyTpCall *call = empathy_tp_call_new (channel);
       priv->channel_wrapper = G_OBJECT (call);
+
+      requested = tp_asv_get_boolean (tp_channel_borrow_immutable_properties (
+            channel), TP_PROP_CHANNEL_REQUESTED, NULL);
+
+      if (!requested &&
+          empathy_tp_call_get_status (call) <= EMPATHY_TP_CALL_STATUS_READYING)
+        {
+          /* For incoming calls, we have to wait that the TpCall is ready as
+           * the call-handler rely on it. */
+          priv->ready_handler = g_signal_connect (call,
+              "notify::status", G_CALLBACK (call_status_changed_cb), self);
+          goto out;
+        }
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
