@@ -32,6 +32,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 #include <libnotify/notification.h>
 
@@ -54,6 +55,7 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-notify-manager.h>
 
+#include "empathy-chat-manager.h"
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
 #include "empathy-invite-participant-dialog.h"
@@ -61,6 +63,13 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
+/* Macro to compare guint32 X timestamps, while accounting for wrapping around
+ */
+#define X_EARLIER_OR_EQL(t1, t2) \
+	((t1 <= t2 && ((t2 - t1) < G_MAXUINT32/2))  \
+	  || (t1 >= t2 && (t1 - t2) > (G_MAXUINT32/2)) \
+	)
+
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChatWindow)
 typedef struct {
 	EmpathyChat *current_chat;
@@ -78,10 +87,14 @@ typedef struct {
 	GtkTargetList *contact_targets;
 	GtkTargetList *file_targets;
 
+	EmpathyChatManager *chat_manager;
+	gulong chat_manager_chats_changed_id;
+
 	/* Menu items. */
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
 	GtkAction   *menu_conv_favorite;
+	GtkAction   *menu_conv_always_urgent;
 	GtkAction   *menu_conv_toggle_contacts;
 
 	GtkAction   *menu_edit_cut;
@@ -91,9 +104,13 @@ typedef struct {
 
 	GtkAction   *menu_tabs_next;
 	GtkAction   *menu_tabs_prev;
+	GtkAction   *menu_tabs_undo_close_tab;
 	GtkAction   *menu_tabs_left;
 	GtkAction   *menu_tabs_right;
 	GtkAction   *menu_tabs_detach;
+
+	/* Last user action time we acted upon to show a tab */
+	guint32    x_user_action_time;
 } EmpathyChatWindowPriv;
 
 static GList *chat_windows = NULL;
@@ -384,15 +401,15 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 		"/chats_menubar/menu_contact");
 	orig_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
 
-	if (orig_submenu == NULL || !GTK_WIDGET_VISIBLE (orig_submenu)) {
+	if (orig_submenu == NULL || !gtk_widget_get_visible (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
 		gtk_widget_show (menu);
 	} else {
-		empathy_signal_connect_weak (orig_submenu,
+		tp_g_signal_connect_object (orig_submenu,
 					     "notify::visible",
 					     (GCallback)_submenu_notify_visible_changed_cb,
-					     G_OBJECT (window));
+					     window, 0);
 	}
 }
 
@@ -769,6 +786,13 @@ chat_window_conv_activate_cb (GtkAction         *action,
 		DEBUG ("This room %s favorite", found ? "is" : "is not");
 		gtk_toggle_action_set_active (
 			GTK_TOGGLE_ACTION (priv->menu_conv_favorite), found);
+
+		if (chatroom != NULL)
+			found = empathy_chatroom_is_always_urgent (chatroom);
+
+		gtk_toggle_action_set_active (
+			GTK_TOGGLE_ACTION (priv->menu_conv_always_urgent),
+			found);
 	}
 	gtk_action_set_visible (priv->menu_conv_favorite, is_room);
 
@@ -812,19 +836,38 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 	account = empathy_chat_get_account (priv->current_chat);
 	room = empathy_chat_get_id (priv->current_chat);
 
-	chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
-						  account, room);
-
-	if (chatroom == NULL) {
-		const gchar *name;
-
-		name = empathy_chat_get_name (priv->current_chat);
-		chatroom = empathy_chatroom_new_full (account, room, name, FALSE);
-		empathy_chatroom_manager_add (priv->chatroom_manager, chatroom);
-		g_object_unref (chatroom);
-	}
+	chatroom = empathy_chatroom_manager_ensure_chatroom (
+		     priv->chatroom_manager,
+		     account,
+		     room,
+		     empathy_chat_get_name (priv->current_chat));
 
 	empathy_chatroom_set_favorite (chatroom, active);
+	g_object_unref (chatroom);
+}
+
+static void
+chat_window_always_urgent_toggled_cb (GtkToggleAction   *toggle_action,
+				 EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gboolean               active;
+	TpAccount             *account;
+	const gchar           *room;
+	EmpathyChatroom       *chatroom;
+
+	active = gtk_toggle_action_get_active (toggle_action);
+	account = empathy_chat_get_account (priv->current_chat);
+	room = empathy_chat_get_id (priv->current_chat);
+
+	chatroom = empathy_chatroom_manager_ensure_chatroom (
+		     priv->chatroom_manager,
+		     account,
+		     room,
+		     empathy_chat_get_name (priv->current_chat));
+
+	empathy_chatroom_set_always_urgent (chatroom, active);
+	g_object_unref (chatroom);
 }
 
 static void
@@ -840,7 +883,7 @@ chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
 }
 
 static void
-got_contact_cb (EmpathyTpContactFactory *factory,
+got_contact_cb (TpConnection            *connection,
                 EmpathyContact          *contact,
                 const GError            *error,
                 gpointer                 user_data,
@@ -884,7 +927,6 @@ chat_window_invite_participant_activate_cb (GtkAction         *action,
 
 	if (response == GTK_RESPONSE_ACCEPT) {
 		TpConnection *connection;
-		EmpathyTpContactFactory *factory;
 		const char *id;
 
 		id = empathy_contact_selector_dialog_get_selected (
@@ -892,12 +934,8 @@ chat_window_invite_participant_activate_cb (GtkAction         *action,
 		if (EMP_STR_EMPTY (id)) goto out;
 
 		connection = tp_channel_borrow_connection (channel);
-		factory = empathy_tp_contact_factory_dup_singleton (connection);
-
-		empathy_tp_contact_factory_get_from_id (factory, id,
+		empathy_tp_contact_factory_get_from_id (connection, id,
 			got_contact_cb, tp_chat,  NULL, NULL);
-
-		g_object_unref (factory);
 	}
 
 out:
@@ -1059,6 +1097,14 @@ chat_window_tabs_previous_activate_cb (GtkAction         *action,
 	gtk_notebook_prev_page (GTK_NOTEBOOK (priv->notebook));
 }
 
+static void
+chat_window_tabs_undo_close_tab_activate_cb (GtkAction         *action,
+					     EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	empathy_chat_manager_undo_closed_chat (priv->chat_manager);
+}
+
 static void
 chat_window_tabs_left_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
@@ -1195,7 +1241,9 @@ chat_window_notification_closed_cb (NotifyNotification *notify,
 	EmpathyChatWindowPriv *priv = GET_PRIV (self);
 
 	g_object_unref (notify);
-	priv->notification = NULL;
+	if (priv->notification == notify) {
+		priv->notification = NULL;
+	}
 }
 
 static void
@@ -1209,7 +1257,8 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	const char *body;
 	GdkPixbuf *pixbuf;
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	gboolean res;
+	gboolean res, has_x_canonical_append;
+	NotifyNotification *notification = priv->notification;
 
 	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		return;
@@ -1225,27 +1274,48 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	header = empathy_contact_get_name (sender);
 	body = empathy_message_get_body (message);
 	escaped = g_markup_escape_text (body, -1);
+	has_x_canonical_append = empathy_notify_manager_has_capability (
+		priv->notify_mgr, EMPATHY_NOTIFY_MANAGER_CAP_X_CANONICAL_APPEND);
 
-	if (priv->notification != NULL) {
-		notify_notification_update (priv->notification,
+	if (notification != NULL && !has_x_canonical_append) {
+		/* if the notification server supports x-canonical-append, it is
+		   better to not use notify_notification_update to avoid
+		   overwriting the current notification message */
+		notify_notification_update (notification,
 					    header, escaped, NULL);
 	} else {
-		priv->notification = notify_notification_new (header, escaped, NULL, NULL);
-		notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
+		/* if the notification server supports x-canonical-append,
+		   the hint will be added, so that the message from the
+		   just created notification will be automatically appended
+		   to an existing notification with the same title.
+		   In this way the previous message will not be lost: the new
+		   message will appear below it, in the same notification */
+		notification = notify_notification_new (header, escaped, NULL, NULL);
 
-		empathy_signal_connect_weak (priv->notification, "closed",
-				  G_CALLBACK (chat_window_notification_closed_cb), G_OBJECT (window));
+		if (priv->notification == NULL) {
+			priv->notification = notification;
+		}
+
+		notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
+
+		g_signal_connect (notification, "closed",
+				  G_CALLBACK (chat_window_notification_closed_cb), window);
+
+		if (has_x_canonical_append) {
+			notify_notification_set_hint_string (notification,
+				EMPATHY_NOTIFY_MANAGER_CAP_X_CANONICAL_APPEND, "");
+		}
 	}
 
 	pixbuf = empathy_notify_manager_get_pixbuf_for_notification (priv->notify_mgr,
 		sender, EMPATHY_IMAGE_NEW_MESSAGE);
 
 	if (pixbuf != NULL) {
-		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
+		notify_notification_set_icon_from_pixbuf (notification, pixbuf);
 		g_object_unref (pixbuf);
 	}
 
-	notify_notification_show (priv->notification, NULL);
+	notify_notification_show (notification, NULL);
 
 	g_free (escaped);
 }
@@ -1312,11 +1382,27 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	/* If empathy_chat_is_room () returns TRUE, that means it's a named MUC.
 	 * If empathy_chat_get_remote_contact () returns NULL, that means it's
 	 * an unamed MUC (msn-like).
-	 * In case of a MUC, we set urgency only if the message contains our
-	 * alias. */
+	 * In case of a MUC, we set urgency if either:
+	 *   a) the chatroom's always_urgent property is TRUE
+	 *   b) the message contains our alias
+	 */
 	if (empathy_chat_is_room (chat) ||
 	    empathy_chat_get_remote_contact (chat) == NULL) {
-		needs_urgency = empathy_message_should_highlight (message);
+		TpAccount             *account;
+		const gchar           *room;
+		EmpathyChatroom       *chatroom;
+
+		account = empathy_chat_get_account (chat);
+		room = empathy_chat_get_id (chat);
+
+		chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
+							  account, room);
+
+		if (empathy_chatroom_is_always_urgent (chatroom)) {
+			needs_urgency = TRUE;
+		} else {
+			needs_urgency = empathy_message_should_highlight (message);
+		}
 	} else {
 		needs_urgency = TRUE;
 	}
@@ -1645,7 +1731,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 			if (connection) {
 				empathy_dispatcher_chat_with_contact_id (
-					connection, contact_id, NULL, NULL);
+					connection, contact_id, gtk_get_current_event_time (), NULL, NULL);
 			}
 
 			g_strfreev (strv);
@@ -1667,7 +1753,8 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		}
 
 		/* Added to take care of any outstanding chat events */
-		empathy_chat_window_present_chat (chat);
+		empathy_chat_window_present_chat (chat,
+			EMPATHY_DISPATCHER_NON_USER_ACTION);
 
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
@@ -1720,6 +1807,17 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	}
 }
 
+static void
+chat_window_chat_manager_chats_changed_cb (EmpathyChatManager *chat_manager,
+					   guint num_chats_in_manager,
+					   EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+
+	gtk_action_set_sensitive (priv->menu_tabs_undo_close_tab,
+				  num_chats_in_manager > 0);
+}
+
 static void
 chat_window_finalize (GObject *object)
 {
@@ -1737,7 +1835,6 @@ chat_window_finalize (GObject *object)
 
 	if (priv->notification != NULL) {
 		notify_notification_close (priv->notification, NULL);
-		g_object_unref (priv->notification);
 		priv->notification = NULL;
 	}
 
@@ -1748,6 +1845,13 @@ chat_window_finalize (GObject *object)
 		gtk_target_list_unref (priv->file_targets);
 	}
 
+	if (priv->chat_manager) {
+		g_signal_handler_disconnect (priv->chat_manager,
+					     priv->chat_manager_chats_changed_id);
+		g_object_unref (priv->chat_manager);
+		priv->chat_manager = NULL;
+	}
+
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
@@ -1799,6 +1903,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "ui_manager", &priv->ui_manager,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
 				       "menu_conv_favorite", &priv->menu_conv_favorite,
+				       "menu_conv_always_urgent", &priv->menu_conv_always_urgent,
 				       "menu_conv_toggle_contacts", &priv->menu_conv_toggle_contacts,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
@@ -1806,6 +1911,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_edit_find", &priv->menu_edit_find,
 				       "menu_tabs_next", &priv->menu_tabs_next,
 				       "menu_tabs_prev", &priv->menu_tabs_prev,
+				       "menu_tabs_undo_close_tab", &priv->menu_tabs_undo_close_tab,
 				       "menu_tabs_left", &priv->menu_tabs_left,
 				       "menu_tabs_right", &priv->menu_tabs_right,
 				       "menu_tabs_detach", &priv->menu_tabs_detach,
@@ -1816,6 +1922,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
+			      "menu_conv_always_urgent", "toggled", chat_window_always_urgent_toggled_cb,
 			      "menu_conv_toggle_contacts", "toggled", chat_window_contacts_toggled_cb,
 			      "menu_conv_invite_participant", "activate", chat_window_invite_participant_activate_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
@@ -1826,6 +1933,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_edit_find", "activate", chat_window_find_activate_cb,
 			      "menu_tabs_next", "activate", chat_window_tabs_next_activate_cb,
 			      "menu_tabs_prev", "activate", chat_window_tabs_previous_activate_cb,
+			      "menu_tabs_undo_close_tab", "activate", chat_window_tabs_undo_close_tab_activate_cb,
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
 			      "menu_tabs_right", "activate", chat_window_tabs_right_activate_cb,
 			      "menu_tabs_detach", "activate", chat_window_detach_activate_cb,
@@ -1931,8 +2039,19 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	priv->chats_new_msg = NULL;
 	priv->chats_composing = NULL;
 	priv->current_chat = NULL;
+	priv->notification = NULL;
 
 	priv->notify_mgr = empathy_notify_manager_dup_singleton ();
+
+	priv->chat_manager = empathy_chat_manager_dup_singleton ();
+	priv->chat_manager_chats_changed_id =
+		g_signal_connect (priv->chat_manager, "chats-changed",
+				  G_CALLBACK (chat_window_chat_manager_chats_changed_cb),
+				  window);
+
+	chat_window_chat_manager_chats_changed_cb (priv->chat_manager,
+						   empathy_chat_manager_get_num_chats (priv->chat_manager),
+						   window);
 }
 
 EmpathyChatWindow *
@@ -2075,6 +2194,7 @@ empathy_chat_window_remove_chat (EmpathyChatWindow *window,
 	EmpathyChatWindowPriv *priv;
 	gint                   position;
 	EmpathyContact        *remote_contact;
+	EmpathyChatManager    *chat_manager;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -2092,6 +2212,10 @@ empathy_chat_window_remove_chat (EmpathyChatWindow *window,
 						      chat);
 	}
 
+	chat_manager = empathy_chat_manager_dup_singleton ();
+	empathy_chat_manager_closed_chat (chat_manager, chat);
+	g_object_unref (chat_manager);
+
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (priv->notebook),
 					  GTK_WIDGET (chat));
 	gtk_notebook_remove_page (GTK_NOTEBOOK (priv->notebook), position);
@@ -2195,10 +2319,12 @@ empathy_chat_window_find_chat (TpAccount   *account,
 }
 
 void
-empathy_chat_window_present_chat (EmpathyChat *chat)
+empathy_chat_window_present_chat (EmpathyChat *chat,
+				  gint64 timestamp)
 {
 	EmpathyChatWindow     *window;
 	EmpathyChatWindowPriv *priv;
+	guint32 x_timestamp;
 
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
 
@@ -2209,16 +2335,40 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
 		window = empathy_chat_window_get_default (empathy_chat_is_room (chat));
 		if (!window) {
 			window = empathy_chat_window_new ();
+			gtk_widget_show_all (GET_PRIV (window)->dialog);
 		}
 
 		empathy_chat_window_add_chat (window, chat);
 	}
 
-	priv = GET_PRIV (window);
-	empathy_chat_window_switch_to_chat (window, chat);
-	empathy_window_present (GTK_WINDOW (priv->dialog));
+	/* Don't force the window to show itself when it wasn't
+	 * an action by the user
+	 */
+	if (timestamp == EMPATHY_DISPATCHER_NON_USER_ACTION)
+		return;
 
- 	gtk_widget_grab_focus (chat->input_text_view);
+	priv = GET_PRIV (window);
+
+	if (timestamp == EMPATHY_DISPATCHER_CURRENT_TIME) {
+		x_timestamp = GDK_CURRENT_TIME;
+	} else {
+		x_timestamp = CLAMP (timestamp, 0, G_MAXUINT32);
+		/* Don't present or switch tab if the action was earlier than the
+		 * last actions X time, accounting for overflow and the first ever
+		* presentation */
+
+		if (priv->x_user_action_time != 0
+			&& X_EARLIER_OR_EQL (x_timestamp, priv->x_user_action_time))
+			return;
+
+		priv->x_user_action_time = x_timestamp;
+	}
+
+	empathy_chat_window_switch_to_chat (window, chat);
+	empathy_window_present_with_time (GTK_WINDOW (priv->dialog),
+	  x_timestamp);
+
+	gtk_widget_grab_focus (chat->input_text_view);
 }
 
 void
