@@ -536,9 +536,6 @@ invite_dialog_response_cb (GtkDialog *dialog,
                            EventManagerApproval *approval)
 {
   EmpathyTpChat *tp_chat;
-  TpChannel *channel;
-  TpHandle self_handle;
-  GArray *members;
   gint64 timestamp;
 
   gtk_widget_destroy (GTK_WIDGET (approval->dialog));
@@ -559,15 +556,7 @@ invite_dialog_response_cb (GtkDialog *dialog,
 
   DEBUG ("Muc invitation accepted");
 
-  /* join the room */
-  channel = empathy_tp_chat_get_channel (tp_chat);
-
-  self_handle = tp_channel_group_get_self_handle (channel);
-  members = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
-  g_array_append_val (members, self_handle);
-
-  tp_cli_channel_interface_group_call_add_members (channel, -1, members,
-      "", NULL, NULL, NULL, NULL);
+  /* We'll join the room when handling the channel */
 
   timestamp = gtk_get_current_event_time ();
   if (timestamp == GDK_CURRENT_TIME)
@@ -576,8 +565,6 @@ invite_dialog_response_cb (GtkDialog *dialog,
   empathy_dispatch_operation_set_user_action_time (approval->operation,
     timestamp);
   empathy_dispatch_operation_approve (approval->operation);
-
-  g_array_free (members, TRUE);
 }
 
 static void
@@ -624,7 +611,7 @@ event_room_channel_process_func (EventPriv *event)
 }
 
 static void
-event_manager_muc_invite_got_contact_cb (EmpathyTpContactFactory *factory,
+event_manager_muc_invite_got_contact_cb (TpConnection *connection,
                                          EmpathyContact *contact,
                                          const GError *error,
                                          gpointer user_data,
@@ -665,7 +652,7 @@ event_manager_muc_invite_got_contact_cb (EmpathyTpContactFactory *factory,
 }
 
 static void
-event_manager_ft_got_contact_cb (EmpathyTpContactFactory *factory,
+event_manager_ft_got_contact_cb (TpConnection *connection,
                                  EmpathyContact *contact,
                                  const GError *error,
                                  gpointer user_data,
@@ -730,20 +717,16 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
                 channel, self_handle, &inviter, NULL, NULL))
             {
               /* We are invited to a room */
-              EmpathyTpContactFactory *factory;
               TpConnection *connection;
 
               DEBUG ("Have been invited to %s. Ask user if he wants to accept",
                   tp_channel_get_identifier (channel));
 
               connection = empathy_tp_chat_get_connection (tp_chat);
-              factory = empathy_tp_contact_factory_dup_singleton (connection);
-
-              empathy_tp_contact_factory_get_from_handle (factory,
+              empathy_tp_contact_factory_get_from_handle (connection,
                   inviter, event_manager_muc_invite_got_contact_cb,
                   approval, NULL, G_OBJECT (manager));
 
-              g_object_unref (factory);
               return;
             }
 
@@ -781,17 +764,13 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
       TpChannel *channel;
       TpConnection *connection;
       TpHandle handle;
-      EmpathyTpContactFactory *factory;
 
       channel = empathy_dispatch_operation_get_channel (operation);
       handle = tp_channel_get_handle (channel, NULL);
 
       connection = tp_channel_borrow_connection (channel);
-      factory = empathy_tp_contact_factory_dup_singleton (connection);
-      empathy_tp_contact_factory_get_from_handle (factory, handle,
+      empathy_tp_contact_factory_get_from_handle (connection, handle,
         event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (manager));
-
-      g_object_unref (factory);
     }
   else
     {
@@ -851,8 +830,7 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
 }
 
 static void
-event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
-    EmpathyContact *contact,
+event_manager_presence_changed_cb (EmpathyContact *contact,
     TpConnectionPresenceType current,
     TpConnectionPresenceType previous,
     EmpathyEventManager *manager)
@@ -872,33 +850,45 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
      TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
     {
       /* contact was online */
-      empathy_conf_get_bool (empathy_conf_get (),
-                      EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT, &preference);
-      if (preference && tp_connection_presence_type_cmp_availability (current,
+      if (tp_connection_presence_type_cmp_availability (current,
           TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0)
         {
           /* someone is logging off */
-          header = g_strdup_printf (_("%s is now offline."),
-            empathy_contact_get_name (contact));
+          empathy_sound_play (empathy_main_window_get (),
+              EMPATHY_SOUND_CONTACT_DISCONNECTED);
 
-          event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-              EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+          empathy_conf_get_bool (empathy_conf_get (),
+              EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT, &preference);
+          if (preference)
+            {
+              header = g_strdup_printf (_("%s is now offline."),
+                  empathy_contact_get_name (contact));
+
+              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+            }
         }
     }
   else
     {
       /* contact was offline */
-      empathy_conf_get_bool (empathy_conf_get (),
-                      EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN, &preference);
       if (preference && tp_connection_presence_type_cmp_availability (current,
           TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
         {
           /* someone is logging in */
-          header = g_strdup_printf (_("%s is now online."),
-            empathy_contact_get_name (contact));
+          empathy_sound_play (empathy_main_window_get (),
+              EMPATHY_SOUND_CONTACT_CONNECTED);
 
-          event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-              EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+          empathy_conf_get_bool (empathy_conf_get (),
+              EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN, &preference);
+          if (preference)
+            {
+              header = g_strdup_printf (_("%s is now online."),
+                  empathy_contact_get_name (contact));
+
+              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+            }
         }
     }
   g_free (header);
@@ -907,6 +897,22 @@ out:
   g_object_unref (idle);
 }
 
+static void
+event_manager_members_changed_cb (EmpathyContactList  *list,
+    EmpathyContact *contact,
+    EmpathyContact *actor,
+    guint reason,
+    gchar *message,
+    gboolean is_member,
+    EmpathyEventManager *manager)
+{
+  if (is_member)
+    g_signal_connect (contact, "presence-changed",
+        G_CALLBACK (event_manager_presence_changed_cb), manager);
+  else
+    g_signal_handlers_disconnect_by_func (contact,
+        event_manager_presence_changed_cb, manager);
+}
 
 static GObject *
 event_manager_constructor (GType type,
@@ -989,12 +995,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
 {
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
-  EmpathyContactMonitor *monitor;
-  EmpathyContactList *list_iface;
-
-  list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
-  monitor = empathy_contact_list_get_monitor (list_iface);
-  g_object_unref (list_iface);
 
   manager->priv = priv;
 
@@ -1004,8 +1004,8 @@ empathy_event_manager_init (EmpathyEventManager *manager)
     G_CALLBACK (event_manager_approve_channel_cb), manager);
   g_signal_connect (priv->contact_manager, "pendings-changed",
     G_CALLBACK (event_manager_pendings_changed_cb), manager);
-  g_signal_connect (monitor, "contact-presence-changed",
-    G_CALLBACK (event_manager_presence_changed_cb), manager);
+  g_signal_connect (priv->contact_manager, "members-changed",
+    G_CALLBACK (event_manager_members_changed_cb), manager);
 }
 
 EmpathyEventManager *
