@@ -43,7 +43,6 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-new-chatroom-dialog.h"
-#include "ephy-spinner.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
@@ -66,6 +65,7 @@ typedef struct {
 	GtkWidget         *label_room;
 	GtkWidget         *entry_room;
 	GtkWidget         *expander_browse;
+	GtkWidget         *hbox_expander;
 	GtkWidget         *throbber;
 	GtkWidget         *treeview;
 	GtkTreeModel      *model;
@@ -202,6 +202,7 @@ empathy_new_chatroom_dialog_show (GtkWindow *parent)
 				       "treeview", &dialog->treeview,
 				       "button_join", &dialog->button_join,
 				       "expander_browse", &dialog->expander_browse,
+				       "hbox_expander", &dialog->hbox_expander,
 				       "label_error_message", &dialog->label_error_message,
 				       "viewport_error", &dialog->viewport_error,
 				       NULL);
@@ -235,13 +236,9 @@ empathy_new_chatroom_dialog_show (GtkWindow *parent)
 	new_chatroom_dialog_model_setup (dialog);
 
 	/* Add throbber */
-	dialog->throbber = ephy_spinner_new ();
-	ephy_spinner_set_size (EPHY_SPINNER (dialog->throbber), GTK_ICON_SIZE_LARGE_TOOLBAR);
-	gtk_widget_show (dialog->throbber);
-	gtk_table_attach (GTK_TABLE (dialog->table_info),
-			  dialog->throbber,
-			  2, 3, 0, 1,
-			  0, 0, 0, 0);
+	dialog->throbber = gtk_spinner_new ();
+	gtk_box_pack_start (GTK_BOX (dialog->hbox_expander), dialog->throbber,
+		TRUE, TRUE, 0);
 
 	/* Account chooser for custom */
 	dialog->account_chooser = empathy_account_chooser_new ();
@@ -491,7 +488,8 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 		dialog->room_list = NULL;
 	}
 
-	ephy_spinner_stop (EPHY_SPINNER (dialog->throbber));
+	gtk_spinner_stop (GTK_SPINNER (dialog->throbber));
+	gtk_widget_hide (dialog->throbber);
 	new_chatroom_dialog_model_clear (dialog);
 
 	if (dialog->account != NULL) {
@@ -553,7 +551,8 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 
 		listing = empathy_tp_roomlist_is_listing (dialog->room_list);
 		if (listing) {
-			ephy_spinner_start (EPHY_SPINNER (dialog->throbber));
+			gtk_spinner_start (GTK_SPINNER (dialog->throbber));
+			gtk_widget_show (dialog->throbber);
 		}
 	}
 
@@ -659,9 +658,11 @@ new_chatroom_dialog_listing_cb (EmpathyTpRoomlist        *room_list,
 
 	/* Update the throbber */
 	if (listing) {
-		ephy_spinner_start (EPHY_SPINNER (dialog->throbber));
+		gtk_spinner_start (GTK_SPINNER (dialog->throbber));
+		gtk_widget_show (dialog->throbber);
 	} else {
-		ephy_spinner_stop (EPHY_SPINNER (dialog->throbber));
+		gtk_spinner_stop (GTK_SPINNER (dialog->throbber));
+		gtk_widget_hide (dialog->throbber);
 	}
 }
 
@@ -730,10 +731,9 @@ new_chatroom_dialog_join (EmpathyNewChatroomDialog *dialog)
 		room_name = g_strdup (room);
 	}
 
-	g_strstrip (room_name);
-
 	DEBUG ("Requesting channel for '%s'", room_name);
-	empathy_dispatcher_join_muc (connection, room_name, NULL, NULL);
+	empathy_dispatcher_join_muc (connection, room_name,
+		gtk_get_current_event_time (), NULL, NULL);
 
 	g_free (room_name);
 }
