@@ -136,6 +136,13 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
       empathy_chat_window_present_chat (chat,
           empathy_dispatch_operation_get_user_action_time (operation));
 
+      if (empathy_tp_chat_is_invited (tp_chat))
+        {
+          /* We have been invited to the room. Add ourself as member as this
+           * channel has been approved. */
+          empathy_tp_chat_join (tp_chat);
+        }
+
       empathy_dispatch_operation_claim (operation);
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
